/*******************************************************************************
* File Name: Debounce.c  
* Version 1.50
*
* Description:
*  The PWM User Module consist of an 8 or 16-bit counter with two 8 or 16-bit
*  comparitors. Each instance of this user module is capable of generating
*  two PWM outputs with the same period. The pulse width is selectable between
*  1 and 255/65535. The period is selectable between 2 and 255/65536 clocks. 
*  The compare value output may be configured to be active when the present 
*  counter is less than or less than/equal to the compare value.
*  A terminal count output is also provided. It generates a pulse one clock
*  width wide when the counter is equal to zero.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "Debounce.h"

uint8 Debounce_initVar = 0u;

/*******************************************************************************
* Function Name: Debounce_Start
********************************************************************************
*
* Summary:
*  The start function initializes the pwm with the default values, the 
*  enables the counter to begin counting.  It does not enable interrupts,
*  the EnableInt command should be called if interrupt generation is required.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  Debounce_initVar: Is modified when this function is called for the first 
*   time. Is used to ensure that initialization happens only once.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(Debounce_initVar == 0u)
    {
        Debounce_Init();
        Debounce_initVar = 1u;
    }
    Debounce_Enable();

}


/*******************************************************************************
* Function Name: Debounce_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the 
*  customizer of the component placed onto schematic. Usually called in 
*  Debounce_Start().
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_Init(void) 
{
    #if (Debounce_UsingFixedFunction || Debounce_UseControl)
        uint8 ctrl;
    #endif
    
   #if (Debounce_UsingFixedFunction)
        /* You are allowed to write the compare value (FF only) */
        Debounce_CONTROL |= Debounce_CFG0_MODE;
        #if (Debounce_DeadBand2_4)
            Debounce_CONTROL |= Debounce_CFG0_DB;
        #endif
		
		/* Set the default Compare Mode */
		#if(Debounce_PSOC3_ES2 || Debounce_PSOC5_ES1)
			ctrl = Debounce_CONTROL2 & ~Debounce_CTRL_CMPMODE1_MASK;
			Debounce_CONTROL2 = ctrl | Debounce_DEFAULT_COMPARE1_MODE;
		#endif
		#if(Debounce_PSOC3_ES3 || Debounce_PSOC5_ES2)
			ctrl = Debounce_CONTROL3 & ~Debounce_CTRL_CMPMODE1_MASK;
			Debounce_CONTROL3 = ctrl | Debounce_DEFAULT_COMPARE1_MODE;
		#endif
        
    #elif (Debounce_UseControl)
        /* Set the default compare mode defined in the parameter */
        ctrl = Debounce_CONTROL & ~Debounce_CTRL_CMPMODE2_MASK & ~Debounce_CTRL_CMPMODE1_MASK;
        Debounce_CONTROL = ctrl | Debounce_DEFAULT_COMPARE2_MODE | Debounce_DEFAULT_COMPARE1_MODE;
    #endif 
        
    #if (!Debounce_UsingFixedFunction)
        #if (Debounce_Resolution == 8)
            /* Set FIFO 0 to 1 byte register for period*/
            Debounce_AUX_CONTROLDP0 |= (Debounce_AUX_CTRL_FIFO0_CLR);
        #else /* (Debounce_Resolution == 16)*/
            /* Set FIFO 0 to 1 byte register for period */
            Debounce_AUX_CONTROLDP0 |= (Debounce_AUX_CTRL_FIFO0_CLR);
            Debounce_AUX_CONTROLDP1 |= (Debounce_AUX_CTRL_FIFO0_CLR);
        #endif
    #endif
        
    Debounce_WritePeriod(Debounce_INIT_PERIOD_VALUE);
    Debounce_WriteCounter(Debounce_INIT_PERIOD_VALUE);
        
        #if (Debounce_UseOneCompareMode)
            Debounce_WriteCompare(Debounce_INIT_COMPARE_VALUE1);
        #else
            Debounce_WriteCompare1(Debounce_INIT_COMPARE_VALUE1);
            Debounce_WriteCompare2(Debounce_INIT_COMPARE_VALUE2);
        #endif
        
        #if (Debounce_KillModeMinTime)
            Debounce_WriteKillTime(Debounce_MinimumKillTime);
        #endif
        
        #if (Debounce_DeadBandUsed)
            Debounce_WriteDeadTime(Debounce_INIT_DEAD_TIME);
        #endif

    #if (Debounce_UseStatus || Debounce_UsingFixedFunction)
        Debounce_SetInterruptMode(Debounce_INIT_INTERRUPTS_MODE);
    #endif
        
    #if (Debounce_UsingFixedFunction)
        /* Globally Enable the Fixed Function Block chosen */
        Debounce_GLOBAL_ENABLE |= Debounce_BLOCK_EN_MASK;
        /* Set the Interrupt source to come from the status register */
        Debounce_CONTROL2 |= Debounce_CTRL2_IRQ_SEL;
    #else
        #if(Debounce_UseStatus)
            /* Use the interrupt output of the status register for IRQ output */
            Debounce_STATUS_AUX_CTRL |= Debounce_STATUS_ACTL_INT_EN_MASK;
            /* Clear the FIFO to enable the Debounce_STATUS_FIFOFULL
                   bit to be set on FIFO full. */
            Debounce_ClearFIFO();
        #endif
    #endif
}


/*******************************************************************************
* Function Name: Debounce_Enable
********************************************************************************
*
* Summary: 
*  Enables the PWM block operation
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Side Effects: 
*  This works only if software enable mode is chosen
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_Enable(void) 
{
    #if (Debounce_UseControl || Debounce_UsingFixedFunction)
        Debounce_CONTROL |= Debounce_CTRL_ENABLE;
    #endif
}


/*******************************************************************************
* Function Name: Debounce_Stop
********************************************************************************
*
* Summary:
*  The stop function halts the PWM, but does not change any modes or disable
*  interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects:
*  If the Enable mode is set to Hardware only then this function
*  has no effect on the operation of the PWM
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_Stop(void) 
{
    #if (Debounce_UseControl || Debounce_UsingFixedFunction)
        Debounce_CONTROL &= ~Debounce_CTRL_ENABLE;
    #endif
}


#if (Debounce_UseOneCompareMode)
#if (Debounce_CompareMode1SW)
/*******************************************************************************
* Function Name: Debounce_SetCompareMode
********************************************************************************
* 
* Summary:
*  This function writes the Compare Mode for the pwm output when in Dither mode,
*  Center Align Mode or One Output Mode.
*
* Parameters:
*  comparemode:  The new compare mode for the PWM output. Use the compare types
*                defined in the H file as input arguments.
*
* Return:
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_SetCompareMode(uint8 comparemode) 
{
    #if(Debounce_UsingFixedFunction)
	    #if(Debounce_PSOC3_ES2 || Debounce_PSOC5_ES1)
			uint8 comparemodemasked = (comparemode << Debounce_CTRL_CMPMODE1_SHIFT);
			Debounce_CONTROL2 &= ~Debounce_CTRL_CMPMODE1_MASK; /*Clear Existing Data */
			Debounce_CONTROL2 |= comparemodemasked;  
		#endif
		
	    #if(Debounce_PSOC3_ES3 || Debounce_PSOC5_ES2)
		    uint8 comparemodemasked = (comparemode << Debounce_CTRL_CMPMODE1_SHIFT);
            Debounce_CONTROL3 &= ~Debounce_CTRL_CMPMODE1_MASK; /*Clear Existing Data */
            Debounce_CONTROL3 |= comparemodemasked;  	
		#endif
                
    #elif (Debounce_UseControl)
        uint8 comparemode1masked = (comparemode << Debounce_CTRL_CMPMODE1_SHIFT) & Debounce_CTRL_CMPMODE1_MASK;
        uint8 comparemode2masked = (comparemode << Debounce_CTRL_CMPMODE2_SHIFT) & Debounce_CTRL_CMPMODE2_MASK;
        Debounce_CONTROL &= ~(Debounce_CTRL_CMPMODE1_MASK | Debounce_CTRL_CMPMODE2_MASK); /*Clear existing mode */
        Debounce_CONTROL |= (comparemode1masked | comparemode2masked);
        
    #else
        uint8 temp = comparemode;
    #endif
}
#endif /* Debounce_CompareMode1SW */

#else /* UseOneCompareMode */


#if (Debounce_CompareMode1SW)
/*******************************************************************************
* Function Name: Debounce_SetCompareMode1
********************************************************************************
* 
* Summary:
*  This function writes the Compare Mode for the pwm or pwm1 output
*
* Parameters:  
*  comparemode:  The new compare mode for the PWM output. Use the compare types
*                defined in the H file as input arguments.
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_SetCompareMode1(uint8 comparemode) 
{
    uint8 comparemodemasked = (comparemode << Debounce_CTRL_CMPMODE1_SHIFT) & Debounce_CTRL_CMPMODE1_MASK;
    #if(Debounce_UsingFixedFunction)
	    #if(Debounce_PSOC3_ES2 || Debounce_PSOC5_ES1)
			Debounce_CONTROL2 &= Debounce_CTRL_CMPMODE1_MASK; /*Clear existing mode */
			Debounce_CONTROL2 |= comparemodemasked; 
	    #endif
		
		#if(Debounce_PSOC3_ES3 || Debounce_PSOC5_ES2)
		    Debounce_CONTROL3 &= Debounce_CTRL_CMPMODE1_MASK; /*Clear existing mode */
			Debounce_CONTROL3 |= comparemodemasked; 
	    #endif
		
    #elif (Debounce_UseControl)
        Debounce_CONTROL &= Debounce_CTRL_CMPMODE1_MASK; /*Clear existing mode */
        Debounce_CONTROL |= comparemodemasked;
    #endif    
}
#endif /* Debounce_CompareMode1SW */


#if (Debounce_CompareMode2SW)
/*******************************************************************************
* Function Name: Debounce_SetCompareMode2
********************************************************************************
* 
* Summary:
*  This function writes the Compare Mode for the pwm or pwm2 output
*
* Parameters:  
*  comparemode:  The new compare mode for the PWM output. Use the compare types
*                defined in the H file as input arguments.
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_SetCompareMode2(uint8 comparemode) 
{
    #if(Debounce_UsingFixedFunction)
        /* Do Nothing because there is no second Compare Mode Register in FF block*/ 
    #elif (Debounce_UseControl)
    uint8 comparemodemasked = (comparemode << Debounce_CTRL_CMPMODE2_SHIFT) & Debounce_CTRL_CMPMODE2_MASK;
    Debounce_CONTROL &= Debounce_CTRL_CMPMODE2_MASK; /*Clear existing mode */
    Debounce_CONTROL |= comparemodemasked;
    #endif    
}
#endif /*Debounce_CompareMode2SW */
#endif /* UseOneCompareMode */


/*******************************************************************************
* Function Name: Debounce_WriteCounter
********************************************************************************
* 
* Summary:
*  This function is used to change the counter value.
*
* Parameters:  
*  counter:  This value may be between 1 and (2^Resolution)-1.   
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_WriteCounter(uint8 counter) 
{
    #if(Debounce_UsingFixedFunction)
        CY_SET_REG16(Debounce_COUNTER_LSB_PTR, (uint16)counter);
    #else
        CY_SET_REG8(Debounce_COUNTER_LSB_PTR, counter);
    #endif
}


#if (!Debounce_UsingFixedFunction)
/*******************************************************************************
* Function Name: Debounce_ReadCounter
********************************************************************************
* 
* Summary:
*  This function returns the current value of the counter.  It doesn't matter
*  if the counter is enabled or running.
*
* Parameters:  
*  void  
*
* Return: 
*  The current value of the counter.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
uint8 Debounce_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    CY_GET_REG8(Debounce_COUNTERCAP_LSB_PTR);
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    return (CY_GET_REG8(Debounce_CAPTURE_LSB_PTR));
}


#if (Debounce_UseStatus)
/*******************************************************************************
* Function Name: Debounce_ClearFIFO
********************************************************************************
* 
* Summary:
*  This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_ClearFIFO(void) 
{
    while(Debounce_ReadStatusRegister() & Debounce_STATUS_FIFONEMPTY)
        Debounce_ReadCapture();
}
#endif /* Debounce_UseStatus */
#endif /* !Debounce_UsingFixedFunction */


/*******************************************************************************
* Function Name: Debounce_WritePeriod
********************************************************************************
* 
* Summary:
*  This function is used to change the period of the counter.  The new period 
*  will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period:  Period value. May be between 1 and (2^Resolution)-1.  A value of 0 
*           will result in the counter remaining at zero.
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_WritePeriod(uint8 period) 
{
    #if(Debounce_UsingFixedFunction)
        CY_SET_REG16(Debounce_PERIOD_LSB_PTR, (uint16)period);
    #else
        CY_SET_REG8(Debounce_PERIOD_LSB_PTR, period);
    #endif
}


#if (Debounce_UseOneCompareMode)
/*******************************************************************************
* Function Name: Debounce_WriteCompare
********************************************************************************
* 
* Summary:
*  This funtion is used to change the compare1 value when the PWM is in Dither
*  mode. The compare output will reflect the new value on the next UDB clock. 
*  The compare output will be driven high when the present counter value is 
*  compared to the compare value based on the compare mode defined in 
*  Dither Mode.
*
* Parameters:  
*  compare:  New compare value.  
*
* Return: 
*  void
*
* Side Effects:
*  This function is only available if the PWM mode parameter is set to
*  Dither Mode, Center Aligned Mode or One Output Mode
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_WriteCompare(uint8 compare) 
{
   CY_SET_REG8(Debounce_COMPARE1_LSB_PTR, compare);
   #if (Debounce_PWMMode == Debounce__B_PWM__DITHER)
        CY_SET_REG8(Debounce_COMPARE2_LSB_PTR, compare+1);
   #endif
}


#else


/*******************************************************************************
* Function Name: Debounce_WriteCompare1
********************************************************************************
* 
* Summary:
*  This funtion is used to change the compare1 value.  The compare output will 
*  reflect the new value on the next UDB clock.  The compare output will be 
*  driven high when the present counter value is less than or less than or 
*  equal to the compare register, depending on the mode.
*
* Parameters:  
*  compare:  New compare value.  
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_WriteCompare1(uint8 compare) 
{
    #if(Debounce_UsingFixedFunction)
        CY_SET_REG16(Debounce_COMPARE1_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG8(Debounce_COMPARE1_LSB_PTR, compare);
    #endif
}


/*******************************************************************************
* Function Name: Debounce_WriteCompare2
********************************************************************************
* 
* Summary:
*  This funtion is used to change the compare value, for compare1 output.  
*  The compare output will reflect the new value on the next UDB clock.  
*  The compare output will be driven high when the present counter value is 
*  less than or less than or equal to the compare register, depending on the 
*  mode.
*
* Parameters:  
*  compare:  New compare value.  
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_WriteCompare2(uint8 compare) 
{
    #if(Debounce_UsingFixedFunction)
        CY_SET_REG16(Debounce_COMPARE2_LSB_PTR, compare);
    #else
        CY_SET_REG8(Debounce_COMPARE2_LSB_PTR, compare);
    #endif
}
#endif /* UseOneCompareMode */


#if (Debounce_DeadBandUsed)
/*******************************************************************************
* Function Name: Debounce_WriteDeadTime
********************************************************************************
* 
* Summary:
*  This function writes the dead-band counts to the corresponding register
*
* Parameters:  
*  deadtime:  Number of counts for dead time 
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_WriteDeadTime(uint8 deadtime) 
{
    /* If using the Dead Band 1-255 mode then just write the register */
    #if(!Debounce_DeadBand2_4)
        CY_SET_REG8(Debounce_DEADBAND_COUNT_PTR, deadtime);
    #else
        /* Otherwise the data has to be masked and offset */        
        /* Clear existing data */
        Debounce_DEADBAND_COUNT &= ~Debounce_DEADBAND_COUNT_MASK; 
	    /* Set new dead time */
        Debounce_DEADBAND_COUNT |= (deadtime << Debounce_DEADBAND_COUNT_SHIFT) & Debounce_DEADBAND_COUNT_MASK; 
    #endif
}


/*******************************************************************************
* Function Name: Debounce_ReadDeadTime
********************************************************************************
* 
* Summary:
*  This function reads the dead-band counts from the corresponding register
*
* Parameters:  
*  void
*
* Return: 
*  Dead Band Counts
*
* Reentrant:
*  Yes
*
*******************************************************************************/
uint8 Debounce_ReadDeadTime(void) 
{
    /* If using the Dead Band 1-255 mode then just read the register */
    #if(!Debounce_DeadBand2_4)
        return (CY_GET_REG8(Debounce_DEADBAND_COUNT_PTR));
    #else
        /* Otherwise the data has to be masked and offset */
        return ((Debounce_DEADBAND_COUNT & Debounce_DEADBAND_COUNT_MASK) >> Debounce_DEADBAND_COUNT_SHIFT);
    #endif
}
#endif /* DeadBandUsed */

