/*******************************************************************************
* File Name: Debounce_PM.c
* Version 1.50
*
* Description:
*  This file provides the power management source code to API for the
*  PWM.
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/
#include "cytypes.h"
#include "Debounce.h"

static Debounce_backupStruct Debounce_backup;


/*******************************************************************************
* Function Name: Debounce_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration of the component.
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Debounce_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void Debounce_SaveConfig(void)
{
        #if(!Debounce_UsingFixedFunction)
        
            #if(!Debounce_PWMModeIsCenterAligned)
                Debounce_backup.periodValue = Debounce_ReadPeriod();
            #endif
            
            #if(Debounce_DeadBand2_4)
                Debounce_backup.deadBandValue = Debounce_ReadDeadTime();
            #endif
          
            Debounce_backup.counterValue = Debounce_ReadCounter();           
            
            #if(Debounce_UseControl)
                Debounce_backup.control = Debounce_ReadControlRegister();
            #endif 
            
        #endif
     
}


/*******************************************************************************
* Function Name: Debounce_RestoreConfig
********************************************************************************
* 
* Summary:
*  Restores the current user configuration of the component.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Debounce_backup:  Variables of this global structure are used to restore 
*  the values of non retention registers on wakeup from sleep mode.
*
* Reentrant:
*  Yes.
*
*******************************************************************************/
void Debounce_RestoreConfig(void) 
{
        #if(!Debounce_UsingFixedFunction)
        
            #if(!Debounce_PWMModeIsCenterAligned)
                Debounce_WritePeriod(Debounce_backup.periodValue);
            #endif
            
            #if(Debounce_DeadBand2_4)
                Debounce_WriteDeadTime(Debounce_backup.deadBandValue);
            #endif
            
            #if(Debounce_UseControl)
                Debounce_WriteControlRegister(Debounce_backup.control);
            #endif
            
            Debounce_WriteCounter(Debounce_backup.counterValue);          
            
        #endif
}


/*******************************************************************************
* Function Name: Debounce_Sleep
********************************************************************************
* 
* Summary:
*  Disables block's operation and saves the user configuration. Should be called 
*  just prior to entering sleep.
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Debounce_backup.pwmEnable:  Is modified depending on the enable state
*  of the block before entering sleep mode.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void Debounce_Sleep(void)
{
    #if(Debounce_UseControl ||Debounce_UsingFixedFunction)
        if(Debounce_CTRL_ENABLE == (Debounce_CONTROL & Debounce_CTRL_ENABLE))
        {
            /*Component is enabled */
            Debounce_backup.pwmEnable = 1u;
        }
        else
        {
            /* Component is disabled */
            Debounce_backup.pwmEnable = 0u;
        }
    #endif
    /* Stop component */
    Debounce_Stop();
    
    /* Save registers configuration */
    Debounce_SaveConfig();
}


/*******************************************************************************
* Function Name: Debounce_Wakeup
********************************************************************************
* 
* Summary:
*  Restores and enables the user configuration. Should be called just after 
*  awaking from sleep.
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Debounce_backup.pwmEnable:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Debounce_Wakeup(void) 
{
     /* Restore registers values */
    Debounce_RestoreConfig();
    
    if(Debounce_backup.pwmEnable != 0u)
    {
        /* Enable component's operation */
        Debounce_Enable();
    } /* Do nothing if component's block was disabled before */
    
}

/* [] END OF FILE */
