/*******************************************************************************
* File Name: FCounter.c  
* Version 1.50
*
*  Description:
*     The Counter User Module consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "FCounter.h"

uint8 FCounter_initVar = 0u;


/*******************************************************************************
* Function Name: FCounter_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void FCounter_Init(void) 
{
        #if (!FCounter_UsingFixedFunction && !FCounter_ControlRegRemoved)
            uint8 ctrl;
        #endif
        
        #if (FCounter_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            FCounter_CONTROL &= FCounter_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (FCounter_PSOC3_ES2 || FCounter_PSOC5_ES1)
                FCounter_CONTROL2 &= ~FCounter_CTRL_MODE_MASK;
            #endif
            #if (FCounter_PSOC3_ES3 || FCounter_PSOC5_ES2)
                FCounter_CONTROL3 &= ~FCounter_CTRL_MODE_MASK;                
            #endif
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (FCounter_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                FCounter_CONTROL |= 0x04u;
            #endif
            
            /* Set the IRQ to use the status register interrupts */
            FCounter_CONTROL2 |= FCounter_CTRL2_IRQ_SEL;
        #else
            #if(!FCounter_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = FCounter_CONTROL & ~FCounter_CTRL_CMPMODE_MASK;
            FCounter_CONTROL = ctrl | FCounter_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = FCounter_CONTROL & ~FCounter_CTRL_CAPMODE_MASK;
            FCounter_CONTROL = ctrl | FCounter_DEFAULT_CAPTURE_MODE;
            #endif
        #endif 
        
        /* Clear all data in the FIFO's */
        #if (!FCounter_UsingFixedFunction)
            FCounter_ClearFIFO();
        #endif
        
        /* Set Initial values from Configuration */
        FCounter_WritePeriod(FCounter_INIT_PERIOD_VALUE);
        FCounter_WriteCounter(FCounter_INIT_COUNTER_VALUE);
        FCounter_SetInterruptMode(FCounter_INIT_INTERRUPTS_MASK);
        
        #if (!FCounter_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            FCounter_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            FCounter_WriteCompare(FCounter_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            FCounter_STATUS_AUX_CTRL |= FCounter_STATUS_ACTL_INT_EN_MASK;
        #endif
}


/*******************************************************************************
* Function Name: FCounter_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
* Reentrant
*  Yes
*
*******************************************************************************/
void FCounter_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (FCounter_UsingFixedFunction)
        FCounter_GLOBAL_ENABLE |= FCounter_BLOCK_EN_MASK;
        FCounter_GLOBAL_STBY_ENABLE |= FCounter_BLOCK_STBY_EN_MASK;
    #endif   
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!FCounter_ControlRegRemoved || FCounter_UsingFixedFunction)
        FCounter_CONTROL |= FCounter_CTRL_ENABLE;                
    #endif
    
}


/*******************************************************************************
* Function Name: FCounter_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  FCounter_initVar: Is modified when this function is called for the first 
*   time. Is used to ensure that initialization happens only once.
*
* Reentrant
*  No
*
*******************************************************************************/
void FCounter_Start(void) 
{
    if(FCounter_initVar == 0u)
    {
        FCounter_Init();
        
        FCounter_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    FCounter_Enable();        
}


/*******************************************************************************
* Function Name: FCounter_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
* Reentrant
*  Yes
*
*******************************************************************************/
void FCounter_Stop(void) 
{
    /* Disable Counter */
    #if(!FCounter_ControlRegRemoved || FCounter_UsingFixedFunction)
        FCounter_CONTROL &= ~FCounter_CTRL_ENABLE;        
    #endif
    
    /* Globally disable the Fixed Function Block chosen */
    #if (FCounter_UsingFixedFunction)
        FCounter_GLOBAL_ENABLE &= ~FCounter_BLOCK_EN_MASK;
        FCounter_GLOBAL_STBY_ENABLE &= ~FCounter_BLOCK_STBY_EN_MASK;
    #endif
}


/*******************************************************************************
* Function Name: FCounter_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void FCounter_SetInterruptMode(uint8 interruptsMask) 
{
    FCounter_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: FCounter_GetInterruptSource
********************************************************************************
* Summary:
* Returns the status register with data about the interrupt source.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint8): Status Register Bit-Field of interrupt source(s)
*
* Side Effects:  The Status register may be clear on read and all interrupt
*                sources must be handled.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint8 FCounter_GetInterruptSource(void) 
{
    return FCounter_STATUS;
}


/*******************************************************************************
* Function Name: FCounter_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint8   FCounter_ReadStatusRegister(void) 
{
    return FCounter_STATUS;
}


#if(!FCounter_ControlRegRemoved)
/*******************************************************************************
* Function Name: FCounter_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint8   FCounter_ReadControlRegister(void) 
{
    return FCounter_CONTROL;
}


/*******************************************************************************
* Function Name: FCounter_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*   
* Reentrant
*  Yes
*
*******************************************************************************/
void    FCounter_WriteControlRegister(uint8 control) 
{
    FCounter_CONTROL = control;
}

#endif  /* (!FCounter_ControlRegRemoved) */


/*******************************************************************************
* Function Name: FCounter_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
* Reentrant
*  Yes
*
*******************************************************************************/
void FCounter_WriteCounter(uint32 counter) 
{
    #if(FCounter_UsingFixedFunction)
        CY_SET_REG16(FCounter_COUNTER_LSB_PTR, (uint16)counter);
    #else
        CY_SET_REG32(FCounter_COUNTER_LSB_PTR, counter);
    #endif
}


/*******************************************************************************
* Function Name: FCounter_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) The present value of the counter.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 FCounter_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    CY_GET_REG8(FCounter_COUNTER_LSB_PTR);
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    return (CY_GET_REG32(FCounter_STATICCOUNT_LSB_PTR));
}


/*******************************************************************************
* Function Name: FCounter_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint32) Present Capture value.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 FCounter_ReadCapture(void) 
{
   return ( CY_GET_REG32(FCounter_STATICCOUNT_LSB_PTR) );  
}


/*******************************************************************************
* Function Name: FCounter_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint32) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void FCounter_WritePeriod(uint32 period) 
{
    #if(FCounter_UsingFixedFunction)
        CY_SET_REG16(FCounter_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG32(FCounter_PERIOD_LSB_PTR,period);
    #endif
}


/*******************************************************************************
* Function Name: FCounter_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) Present period value.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 FCounter_ReadPeriod(void) 
{
   return ( CY_GET_REG32(FCounter_PERIOD_LSB_PTR));
}


#if (!FCounter_UsingFixedFunction)
/*******************************************************************************
* Function Name: FCounter_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void FCounter_WriteCompare(uint32 compare) 
{
    #if(FCounter_UsingFixedFunction)
        CY_SET_REG16(FCounter_COMPARE_LSB_PTR,(uint16)compare);
    #else
        CY_SET_REG32(FCounter_COMPARE_LSB_PTR,compare);
    #endif
}


/*******************************************************************************
* Function Name: FCounter_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint32) Present compare value.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 FCounter_ReadCompare(void) 
{
   return ( CY_GET_REG32(FCounter_COMPARE_LSB_PTR));
}


#if (FCounter_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: FCounter_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void FCounter_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    FCounter_CONTROL &= ~FCounter_CTRL_CMPMODE_MASK;
    
    /* Write the new setting */
    FCounter_CONTROL |= (compareMode << FCounter_CTRL_CMPMODE0_SHIFT);
}
#endif  /* (FCounter_COMPARE_MODE_SOFTWARE) */


#if (FCounter_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: FCounter_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void FCounter_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    FCounter_CONTROL &= ~FCounter_CTRL_CAPMODE_MASK;
    
    /* Write the new setting */
    FCounter_CONTROL |= (captureMode << FCounter_CTRL_CAPMODE0_SHIFT);
}
#endif  /* (FCounter_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: FCounter_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
* Reentrant
*  Yes
*
*******************************************************************************/
void FCounter_ClearFIFO(void) 
{

    while(FCounter_ReadStatusRegister() & FCounter_STATUS_FIFONEMP)
    {
        FCounter_ReadCapture();
    }

}
#endif  /* (!FCounter_UsingFixedFunction) */


/* [] END OF FILE */

