/*******************************************************************************
* File Name: FCounter_PM.c  
* Version 1.50
*
*  Description:
*     This file provides the power management source code to API for the
*     Counter.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "FCounter.h"

static FCounter_backupStruct FCounter_backup;


/*******************************************************************************
* Function Name: FCounter_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  FCounter_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
* Reentrant:
*    No
*
*******************************************************************************/
void FCounter_SaveConfig(void)
{
    #if (! FCounter_UsingFixedFunction)    
        #if(!FCounter_ControlRegRemoved)
            FCounter_backup.control = FCounter_ReadControlRegister();
        #endif
        
        FCounter_backup.counterUdb = FCounter_ReadCounter();
        FCounter_backup.captureValue = FCounter_ReadCapture();
    #endif
}


/*******************************************************************************
* Function Name: FCounter_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  FCounter_backup:  Variables of this global structure are used to restore 
*  the values of non retention registers on wakeup from sleep mode.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void FCounter_RestoreConfig(void) 
{      
    #if (! FCounter_UsingFixedFunction)     
        #if(!FCounter_ControlRegRemoved)
            FCounter_WriteControlRegister(FCounter_backup.control);
        #endif
        
        FCounter_WriteCounter(FCounter_backup.counterUdb);        
        CY_SET_REG32 (FCounter_STATICCOUNT_LSB_PTR, FCounter_backup.captureValue);
    #endif
}


/*******************************************************************************
* Function Name: FCounter_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  FCounter_backup.enableState:  Is modified depending on the enable state
*  of the block before entering sleep mode.
*
* Reentrant:
*  No
*
*******************************************************************************/
void FCounter_Sleep(void)
{
    #if(!FCounter_ControlRegRemoved)
        /* Save Counter's enable state */
        if(FCounter_CTRL_ENABLE == (FCounter_CONTROL & FCounter_CTRL_ENABLE))
        {
            /* Counter is enabled */
            FCounter_backup.enableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            FCounter_backup.enableState = 0u;
        }
    #endif
    
    FCounter_Stop();
    FCounter_SaveConfig();
}


/*******************************************************************************
* Function Name: FCounter_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  FCounter_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void FCounter_Wakeup(void) 
{
    FCounter_RestoreConfig();
    
    #if(!FCounter_ControlRegRemoved)
        if(FCounter_backup.enableState == 1u)
        {
            /* Enable Counter's operation */
            FCounter_Enable();
    
        } /* Do nothing if Counter was disabled before */    
    #endif
}


/* [] END OF FILE */
