/*******************************************************************************
* File Name: Mixer_L.c  
* Version 1.50
*
* Description:
*  This file provides the source code to the API for the MIXER Component.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "Mixer_L.h"

#if(Mixer_L_MIN_VDDA)
    #include "Mixer_L_bst_clk.h"
#endif /* Mixer_L_MIN_VDDA */

/* Fixed configuration of SC Block only performed once */
uint8 Mixer_L_initVar = 0u;


/*******************************************************************************
* Function Name: Mixer_L_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the 
*  customizer of the component placed onto schematic. Usually called in 
*  Mixer_L_Start().
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Mixer_L_Init(void) 
{
    uint8 tempCR2 = 0u;

    /* Configure SC Block based on selected Mixer type */
    if (Mixer_L_MIXER_TYPE == Mixer_L_CTMIXER) /* Continuous Time (up) mixer */
    {
        /* SC Block mode -  - SCxx_CR0[3:1] */
        Mixer_L_CR0_REG = Mixer_L_MODE_CTMIXER;   

        /* SC Block CR1 */ 
        Mixer_L_CR1_REG = (Mixer_L_COMP_4P35PF  |
                                Mixer_L_DIV2_DISABLE |
                                Mixer_L_GAIN_0DB);                                 

        /* SC Block CR2 */
        tempCR2 = (Mixer_L_BIAS_LOW |
                   Mixer_L_REDC_01 |
                   Mixer_L_GNDVREF_DI);

        /* 
            Set SC Block resistor values based on Local Oscillator being above 
            or below 100kHz - controled by r20_40 (Rmix) bit for CT mode mixer 
        */
        if (Mixer_L_LO_FREQ_FLAG == Mixer_L_LO_LT_100K)
        {
            tempCR2 |= Mixer_L_R20_40B_40K; 
        }
        else
        {
            tempCR2 |= Mixer_L_R20_40B_20K;
        }
        Mixer_L_CR2_REG = tempCR2;
    }
    else
    {   /* Discrete Time (down) mixer */
        /* SC Block mode -  - SCxx_CR0[3:1] */
        Mixer_L_CR0_REG = Mixer_L_MODE_DTMIXER;       

        /* SC Block CR1 */ 
        Mixer_L_CR1_REG = (Mixer_L_COMP_4P35PF  |
                                Mixer_L_DIV2_ENABLE |
                                Mixer_L_GAIN_0DB);

        /* SC Block CR2 */
        tempCR2 = (Mixer_L_BIAS_LOW |Mixer_L_GNDVREF_DI);

        /* 
            Set SC Block resistor values based on Local Oscillator beign above 
            or below 100kHz - set r20_40 (input resitance) and rval (feedback) 
            resistor values seperately for DT mode
        */
        if (Mixer_L_LO_FREQ_FLAG == Mixer_L_LO_LT_100K)
        {
            tempCR2 |= (Mixer_L_R20_40B_40K | Mixer_L_RVAL_40K); 
        }
        else
        {
            tempCR2 |= (Mixer_L_R20_40B_20K | Mixer_L_RVAL_20K);
        }
        Mixer_L_CR2_REG = tempCR2; 
    }/* end if - Mixer Type */

    /* Enable SC Block clocking */
    Mixer_L_CLK_REG |= Mixer_L_CLK_EN;

    /* Set default power */
    Mixer_L_SetPower(Mixer_L_INIT_POWER);
}


/*******************************************************************************
* Function Name: Mixer_L_Enable
********************************************************************************
*  
* Summary: 
*  Enables the Mixer block operation
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Mixer_L_Enable(void) 
{
    /* Enable power to SC block in active mode */
    Mixer_L_PM_ACT_CFG_REG |= Mixer_L_ACT_PWR_EN;

    /* Enable power to SC block in Alternative active mode */
    Mixer_L_PM_STBY_CFG_REG |= Mixer_L_STBY_PWR_EN;

    /* Enable SC Block boost clock control for low Vdda operation */
    #if(Mixer_L_MIN_VDDA)
        Mixer_L_BSTCLK_REG |= Mixer_L_BST_CLK_EN;  /* enable for Vdda < 2.7V */
        Mixer_L_bst_clk_Enable();
    #endif /* Mixer_L_MIN_VDDA */

    /* PSoC3 ES2 or early, PSoC5 ES1 */
    #if (Mixer_L_PSOC3_ES2 || Mixer_L_PSOC5_ES1)
        /* Enable Pump only if voltage is below threshold */
        if (Mixer_L_CYDEV_VDDA_MV < Mixer_L_VDDA_THRESHOLD_MV)
        {
            Mixer_L_SC_MISC_REG |= Mixer_L_PUMP_FORCE;
        }
    /* PSoC3 ES3 or later, PSoC5 ES2 or later */
    #elif (Mixer_L_PSOC3_ES3 || Mixer_L_PSOC5_ES2)
        /* Enable charge Pump clock for SC block */
        Mixer_L_PUMP_CR1_REG |= Mixer_L_PUMP_CR1_SC_CLKSEL;
        /* Enable Pump only if voltage is below threshold */
        if (Mixer_L_CYDEV_VDDA_MV < Mixer_L_VDDA_THRESHOLD_MV)
        {
            Mixer_L_SC_MISC_REG |= Mixer_L_PUMP_FORCE;
        }
    #endif /* Mixer_L_PSOC3_ES2 || Mixer_L_PSOC5_ES1 */
}


/*******************************************************************************
* Function Name: Mixer_L_Start
********************************************************************************
*
* Summary:
*  The start function initializes the SC block control registers based on the
*  Mixer type selected and and enables power to the SC block
*
* Parameters:  
*  void
*
* Return: 
*  void 
*
* Global variables:
*  Mixer_L_initVar:  Used to check the initial configuration,
*  modified when this function is called for the first time.
*
* Theory: 
*  Full initialization of the SC Block configuration registers is only perfomed 
*  the first time the routine executes - global variable 
*  Mixer_L_initVar is used indicate that the static configuration has 
*  been completed.
*
* Reentrant:
*  No
*
*******************************************************************************/
void Mixer_L_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(Mixer_L_initVar == 0u)
    {
        Mixer_L_Init();
        Mixer_L_initVar = 1u;
    }
    Mixer_L_Enable();
}


/*******************************************************************************
* Function Name: Mixer_L_Stop
********************************************************************************
*
* Summary:
*  Disables power to SC block.
*
* Parameters:  
*  void 
*
* Return: 
*  void 
*
* Reentrant: 
*  Yes
*
*******************************************************************************/
void Mixer_L_Stop(void) 
{
    /* Disable pumps only if only one SC block is in use */
    if (Mixer_L_PM_ACT_CFG_REG == Mixer_L_ACT_PWR_EN)
    {
        Mixer_L_SC_MISC_REG &= ~Mixer_L_PUMP_FORCE;
    }
   
    /* Disble power to the Amp in Active mode template */
    Mixer_L_PM_ACT_CFG_REG &= ~Mixer_L_ACT_PWR_EN;

    /* Disble power to the Amp in Alternative Active mode template */
    Mixer_L_PM_STBY_CFG_REG &= ~Mixer_L_STBY_PWR_EN;

    /* Disable SC Block boost clk control, if used (MinVdda < 2.7V) */
    #if(Mixer_L_MIN_VDDA)
        Mixer_L_BSTCLK_REG &= ~Mixer_L_BST_CLK_EN;
        Mixer_L_bst_clk_Disable();
    #endif /* Mixer_L_MIN_VDDA */   
}


/*******************************************************************************
* Function Name: Mixer_L_SetPower
********************************************************************************
*
* Summary:
*  Set the drive power of the MIXER
*
* Parameters:  
*  power:  Sets power level between (0) and (3) high power
*
* Return: 
*  void 
*
* Reentrant: 
*  Yes
*
*******************************************************************************/
void Mixer_L_SetPower(uint8 power) 
{
    uint8 tmpCR;

    /* Sets drive bits in SC Block Control Register 1:  SCxx_CR[1:0] */    
    tmpCR = Mixer_L_CR1_REG & ~Mixer_L_DRIVE_MASK;
    tmpCR |= (power & Mixer_L_DRIVE_MASK);
    Mixer_L_CR1_REG = tmpCR;  
}


/* [] END OF FILE */
