/*******************************************************************************
* File Name: SysCounter.c  
* Version 1.50
*
*  Description:
*     The Counter User Module consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "SysCounter.h"

uint8 SysCounter_initVar = 0u;


/*******************************************************************************
* Function Name: SysCounter_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void SysCounter_Init(void) 
{
        #if (!SysCounter_UsingFixedFunction && !SysCounter_ControlRegRemoved)
            uint8 ctrl;
        #endif
        
        #if (SysCounter_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            SysCounter_CONTROL &= SysCounter_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (SysCounter_PSOC3_ES2 || SysCounter_PSOC5_ES1)
                SysCounter_CONTROL2 &= ~SysCounter_CTRL_MODE_MASK;
            #endif
            #if (SysCounter_PSOC3_ES3 || SysCounter_PSOC5_ES2)
                SysCounter_CONTROL3 &= ~SysCounter_CTRL_MODE_MASK;                
            #endif
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (SysCounter_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                SysCounter_CONTROL |= 0x04u;
            #endif
            
            /* Set the IRQ to use the status register interrupts */
            SysCounter_CONTROL2 |= SysCounter_CTRL2_IRQ_SEL;
        #else
            #if(!SysCounter_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = SysCounter_CONTROL & ~SysCounter_CTRL_CMPMODE_MASK;
            SysCounter_CONTROL = ctrl | SysCounter_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = SysCounter_CONTROL & ~SysCounter_CTRL_CAPMODE_MASK;
            SysCounter_CONTROL = ctrl | SysCounter_DEFAULT_CAPTURE_MODE;
            #endif
        #endif 
        
        /* Clear all data in the FIFO's */
        #if (!SysCounter_UsingFixedFunction)
            SysCounter_ClearFIFO();
        #endif
        
        /* Set Initial values from Configuration */
        SysCounter_WritePeriod(SysCounter_INIT_PERIOD_VALUE);
        SysCounter_WriteCounter(SysCounter_INIT_COUNTER_VALUE);
        SysCounter_SetInterruptMode(SysCounter_INIT_INTERRUPTS_MASK);
        
        #if (!SysCounter_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            SysCounter_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            SysCounter_WriteCompare(SysCounter_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            SysCounter_STATUS_AUX_CTRL |= SysCounter_STATUS_ACTL_INT_EN_MASK;
        #endif
}


/*******************************************************************************
* Function Name: SysCounter_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
* Reentrant
*  Yes
*
*******************************************************************************/
void SysCounter_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (SysCounter_UsingFixedFunction)
        SysCounter_GLOBAL_ENABLE |= SysCounter_BLOCK_EN_MASK;
        SysCounter_GLOBAL_STBY_ENABLE |= SysCounter_BLOCK_STBY_EN_MASK;
    #endif   
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!SysCounter_ControlRegRemoved || SysCounter_UsingFixedFunction)
        SysCounter_CONTROL |= SysCounter_CTRL_ENABLE;                
    #endif
    
}


/*******************************************************************************
* Function Name: SysCounter_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  SysCounter_initVar: Is modified when this function is called for the first 
*   time. Is used to ensure that initialization happens only once.
*
* Reentrant
*  No
*
*******************************************************************************/
void SysCounter_Start(void) 
{
    if(SysCounter_initVar == 0u)
    {
        SysCounter_Init();
        
        SysCounter_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    SysCounter_Enable();        
}


/*******************************************************************************
* Function Name: SysCounter_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
* Reentrant
*  Yes
*
*******************************************************************************/
void SysCounter_Stop(void) 
{
    /* Disable Counter */
    #if(!SysCounter_ControlRegRemoved || SysCounter_UsingFixedFunction)
        SysCounter_CONTROL &= ~SysCounter_CTRL_ENABLE;        
    #endif
    
    /* Globally disable the Fixed Function Block chosen */
    #if (SysCounter_UsingFixedFunction)
        SysCounter_GLOBAL_ENABLE &= ~SysCounter_BLOCK_EN_MASK;
        SysCounter_GLOBAL_STBY_ENABLE &= ~SysCounter_BLOCK_STBY_EN_MASK;
    #endif
}


/*******************************************************************************
* Function Name: SysCounter_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void SysCounter_SetInterruptMode(uint8 interruptsMask) 
{
    SysCounter_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: SysCounter_GetInterruptSource
********************************************************************************
* Summary:
* Returns the status register with data about the interrupt source.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint8): Status Register Bit-Field of interrupt source(s)
*
* Side Effects:  The Status register may be clear on read and all interrupt
*                sources must be handled.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint8 SysCounter_GetInterruptSource(void) 
{
    return SysCounter_STATUS;
}


/*******************************************************************************
* Function Name: SysCounter_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint8   SysCounter_ReadStatusRegister(void) 
{
    return SysCounter_STATUS;
}


#if(!SysCounter_ControlRegRemoved)
/*******************************************************************************
* Function Name: SysCounter_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint8   SysCounter_ReadControlRegister(void) 
{
    return SysCounter_CONTROL;
}


/*******************************************************************************
* Function Name: SysCounter_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*   
* Reentrant
*  Yes
*
*******************************************************************************/
void    SysCounter_WriteControlRegister(uint8 control) 
{
    SysCounter_CONTROL = control;
}

#endif  /* (!SysCounter_ControlRegRemoved) */


/*******************************************************************************
* Function Name: SysCounter_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
* Reentrant
*  Yes
*
*******************************************************************************/
void SysCounter_WriteCounter(uint32 counter) 
{
    #if(SysCounter_UsingFixedFunction)
        CY_SET_REG16(SysCounter_COUNTER_LSB_PTR, (uint16)counter);
    #else
        CY_SET_REG32(SysCounter_COUNTER_LSB_PTR, counter);
    #endif
}


/*******************************************************************************
* Function Name: SysCounter_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) The present value of the counter.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 SysCounter_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    CY_GET_REG8(SysCounter_COUNTER_LSB_PTR);
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    return (CY_GET_REG32(SysCounter_STATICCOUNT_LSB_PTR));
}


/*******************************************************************************
* Function Name: SysCounter_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint32) Present Capture value.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 SysCounter_ReadCapture(void) 
{
   return ( CY_GET_REG32(SysCounter_STATICCOUNT_LSB_PTR) );  
}


/*******************************************************************************
* Function Name: SysCounter_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint32) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void SysCounter_WritePeriod(uint32 period) 
{
    #if(SysCounter_UsingFixedFunction)
        CY_SET_REG16(SysCounter_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG32(SysCounter_PERIOD_LSB_PTR,period);
    #endif
}


/*******************************************************************************
* Function Name: SysCounter_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) Present period value.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 SysCounter_ReadPeriod(void) 
{
   return ( CY_GET_REG32(SysCounter_PERIOD_LSB_PTR));
}


#if (!SysCounter_UsingFixedFunction)
/*******************************************************************************
* Function Name: SysCounter_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void SysCounter_WriteCompare(uint32 compare) 
{
    #if(SysCounter_UsingFixedFunction)
        CY_SET_REG16(SysCounter_COMPARE_LSB_PTR,(uint16)compare);
    #else
        CY_SET_REG32(SysCounter_COMPARE_LSB_PTR,compare);
    #endif
}


/*******************************************************************************
* Function Name: SysCounter_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint32) Present compare value.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 SysCounter_ReadCompare(void) 
{
   return ( CY_GET_REG32(SysCounter_COMPARE_LSB_PTR));
}


#if (SysCounter_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: SysCounter_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void SysCounter_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    SysCounter_CONTROL &= ~SysCounter_CTRL_CMPMODE_MASK;
    
    /* Write the new setting */
    SysCounter_CONTROL |= (compareMode << SysCounter_CTRL_CMPMODE0_SHIFT);
}
#endif  /* (SysCounter_COMPARE_MODE_SOFTWARE) */


#if (SysCounter_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: SysCounter_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void SysCounter_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    SysCounter_CONTROL &= ~SysCounter_CTRL_CAPMODE_MASK;
    
    /* Write the new setting */
    SysCounter_CONTROL |= (captureMode << SysCounter_CTRL_CAPMODE0_SHIFT);
}
#endif  /* (SysCounter_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: SysCounter_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
* Reentrant
*  Yes
*
*******************************************************************************/
void SysCounter_ClearFIFO(void) 
{

    while(SysCounter_ReadStatusRegister() & SysCounter_STATUS_FIFONEMP)
    {
        SysCounter_ReadCapture();
    }

}
#endif  /* (!SysCounter_UsingFixedFunction) */


/* [] END OF FILE */

