/*******************************************************************************
* File Name: SysCounter_PM.c  
* Version 1.50
*
*  Description:
*     This file provides the power management source code to API for the
*     Counter.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "SysCounter.h"

static SysCounter_backupStruct SysCounter_backup;


/*******************************************************************************
* Function Name: SysCounter_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SysCounter_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
* Reentrant:
*    No
*
*******************************************************************************/
void SysCounter_SaveConfig(void)
{
    #if (! SysCounter_UsingFixedFunction)    
        #if(!SysCounter_ControlRegRemoved)
            SysCounter_backup.control = SysCounter_ReadControlRegister();
        #endif
        
        SysCounter_backup.counterUdb = SysCounter_ReadCounter();
        SysCounter_backup.captureValue = SysCounter_ReadCapture();
    #endif
}


/*******************************************************************************
* Function Name: SysCounter_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SysCounter_backup:  Variables of this global structure are used to restore 
*  the values of non retention registers on wakeup from sleep mode.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SysCounter_RestoreConfig(void) 
{      
    #if (! SysCounter_UsingFixedFunction)     
        #if(!SysCounter_ControlRegRemoved)
            SysCounter_WriteControlRegister(SysCounter_backup.control);
        #endif
        
        SysCounter_WriteCounter(SysCounter_backup.counterUdb);        
        CY_SET_REG32 (SysCounter_STATICCOUNT_LSB_PTR, SysCounter_backup.captureValue);
    #endif
}


/*******************************************************************************
* Function Name: SysCounter_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SysCounter_backup.enableState:  Is modified depending on the enable state
*  of the block before entering sleep mode.
*
* Reentrant:
*  No
*
*******************************************************************************/
void SysCounter_Sleep(void)
{
    #if(!SysCounter_ControlRegRemoved)
        /* Save Counter's enable state */
        if(SysCounter_CTRL_ENABLE == (SysCounter_CONTROL & SysCounter_CTRL_ENABLE))
        {
            /* Counter is enabled */
            SysCounter_backup.enableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            SysCounter_backup.enableState = 0u;
        }
    #endif
    
    SysCounter_Stop();
    SysCounter_SaveConfig();
}


/*******************************************************************************
* Function Name: SysCounter_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SysCounter_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SysCounter_Wakeup(void) 
{
    SysCounter_RestoreConfig();
    
    #if(!SysCounter_ControlRegRemoved)
        if(SysCounter_backup.enableState == 1u)
        {
            /* Enable Counter's operation */
            SysCounter_Enable();
    
        } /* Do nothing if Counter was disabled before */    
    #endif
}


/* [] END OF FILE */
